<?php
/**
 * @file
 * Shepherd Server callbacks for client update, client test ping, and reporting.
 */

/**
 * callback that waits for a client http POST.
 * From POST, we build a node out of the info and 
 * make it available to the shepherd server.
 *
 *
 * @return
 *   None presently, maybe it should return result of drupal_execute?
 */
function shepherd_server_receive_client_update() {
  // might not need these any longer.
  global $user;
  $user = user_load(array(uid => 1));

  $sheep = (object) array('type' => 'sheep');
  
  $client   = $_POST['client_name'];
  $vhost    = $_POST['client_baseurl'];
  $ip       = $_POST['client_ip'];
  $hashkey  = _shepherd_sever_generate_hashkey($client, $vhost, $ip);
  $data     = $_POST['data'];
  
  // does the client already exist?
  $existing_node_id = shepherd_client_check($hashkey);
  if ($existing_node_id) {
    $sheep = sheep_load($existing_node_id);
  }
  
  $sheep->vhost     = $vhost;
  $sheep->ip        = $ip;
  $sheep->sitename  = $client;
  $sheep->hashkey   = $hashkey;
  $sheep->update_status = $data;  
  
  
  // save the sheep
  $result = shepherd_sheep_save($sheep);
  if ($result) {
    watchdog('shepherd', "Received sheep from client: %vhost", array('%vhost' => $vhost));
  }
}

/**
 * Check white list for allowed connections and
 * respond with header and text
 *
 * @return
 *   None, returns 200 or 401 http headers to be read by client.
 */
function shepherd_server_client_ping_response() {
  $connecting_ip = $_SERVER['REMOTE_ADDR'];
  $whitelist = explode("\r\n", variable_get('shepherd_ip_whitelist', ""));
  $allowed = in_array($connecting_ip, $whitelist);
 
  if ($allowed) {
     header('HTTP/1.1 200 OK');
     echo 'allowed';
  }
  else {
     header('HTTP/1.1 401 Unauthorized');
     echo 'not allowed';
  }
  
  exit;
}

/**
 * Look for existing sheep nodes by the hashkey
 * @param $hashkey
 *   key created from the ip address, vhost, and site name of a client
 *
 * @return
 *   nid of found sheep
 *   FALSE if hashkey not found.
 */
function shepherd_client_check($hashkey) {
  $query = "SELECT nid FROM {shepherd_sheep} WHERE hashkey = '%s'";
  
  $result = db_query($query, $hashkey);
  $nid = db_result($result);
  if ($nid) {
    watchdog('shepherd', "found existing sheep: ". print_r($result, true));
    return $nid;
  }
  
  return FALSE;
}

/**
 * display callback for the Shepherd flock report
 *
 * @return
 *   themed report page
 */
function shepherd_server_flock_report() {
  $report = "<div id='flock-report'>";

  $sql = "SELECT nid FROM {shepherd_sheep} ORDER BY sitename ASC";
  $result = db_query($sql);
  
  while ($row = db_fetch_array($result)) {
    $sheep = sheep_load($row['nid']);
    $report .= theme('flock_report', $sheep);
  }

  $report .= "</div>";
  
  return $report;
}

/**
 * generate a key that can be used to quickly look up this node.
 * possibly add index via install hook to this table after content type import.
 *
 *
 */
function _shepherd_sever_generate_hashkey($client_name, $url, $ip) {
  return md5($client_name ."###". $url ."###". $ip);
}

/**
 * Save a sheep record to storage.
 * @param $sheep
 *   the sheep item to save
 *
 */
function shepherd_sheep_save(&$sheep) {
  if (empty($sheep->nid)) {
    // insert
    $now = time();
    $sheep->created = $now;
    $sheep->changed = $now;
    $result = drupal_write_record('shepherd_sheep', $sheep);
  }
  else {
    //update
    $sheep->changed = time();
    $result = drupal_write_record('shepherd_sheep', $sheep, array('nid'));
  }
  
  if ($result) {
    $msg = "Sheep '%nid', was " . ($result === SAVED_NEW ? "Inserted" : "Updated");
    watchdog('shepherd', $msg, array('%nid' => $sheep->nid));
  }
  else {
    watchdog('shepherd', "Sheep data could not be saved.");
  }
  
  return $result;
}